@@ -29,7 +29,7 @@ func TestWithMetrics(t *testing.T) {
 	methodNames := make(map[string]struct{})
 	wv := reflect.ValueOf(w)
 	wt := reflect.TypeOf(w)
-	for i := 0; i < wt.NumMethod(); i++ {
+	for i := range wt.NumMethod() {
 		methodNames[wt.Method(i).Name] = struct{}{}
 	}
 
@@ -133,6 +133,10 @@ func TestWithMetrics(t *testing.T) {
 			key:        "datastore.registration_entry.fetch",
 			methodName: "FetchRegistrationEntry",
 		},
+		{
+			key:        "datastore.registration_entry.fetch",
+			methodName: "FetchRegistrationEntries",
+		},
 		{
 			key:        "datastore.registration_entry_event.fetch",
 			methodName: "FetchRegistrationEntryEvent",
@@ -250,7 +254,6 @@ func TestWithMetrics(t *testing.T) {
 			methodName: "ListCAJournalsForTesting",
 		},
 	} {
-		tt := tt
 		methodType, ok := wt.MethodByName(tt.methodName)
 		require.True(t, ok, "method %q does not exist on DataStore interface", tt.methodName)
 		methodValue := wv.Method(methodType.Index)
@@ -270,7 +273,7 @@ func TestWithMetrics(t *testing.T) {
 			}
 			out := methodValue.Call(args)
 			require.Len(t, out, numOut)
-			for i := 0; i < numOut-1; i++ {
+			for i := range numOut - 1 {
 				mv := methodValue.Type().Out(i)
 				switch v := reflect.ValueOf(mv); v.Kind() {
 				case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
@@ -433,6 +436,10 @@ func (ds *fakeDataStore) FetchRegistrationEntry(context.Context, string) (*commo
 	return &common.RegistrationEntry{}, ds.err
 }
 
+func (ds *fakeDataStore) FetchRegistrationEntries(context.Context, []string) (map[string]*common.RegistrationEntry, error) {
+	return map[string]*common.RegistrationEntry{}, ds.err
+}
+
 func (ds *fakeDataStore) FetchRegistrationEntryEvent(context.Context, uint) (*datastore.RegistrationEntryEvent, error) {
 	return &datastore.RegistrationEntryEvent{}, ds.err
 }
