@@ -17,6 +17,7 @@ import (
 	"github.com/sirupsen/logrus"
 	"github.com/sirupsen/logrus/hooks/test"
 	"github.com/spiffe/go-spiffe/v2/spiffeid"
+	"github.com/spiffe/spire/pkg/common/catalog"
 	"github.com/spiffe/spire/pkg/common/coretypes/x509certificate"
 	telemetry_server "github.com/spiffe/spire/pkg/common/telemetry/server"
 	"github.com/spiffe/spire/pkg/common/x509util"
@@ -24,6 +25,7 @@ import (
 	"github.com/spiffe/spire/pkg/server/credtemplate"
 	"github.com/spiffe/spire/pkg/server/credvalidator"
 	"github.com/spiffe/spire/pkg/server/datastore"
+	"github.com/spiffe/spire/pkg/server/plugin/credentialcomposer"
 	"github.com/spiffe/spire/pkg/server/plugin/keymanager"
 	"github.com/spiffe/spire/pkg/server/plugin/notifier"
 	"github.com/spiffe/spire/pkg/server/plugin/upstreamauthority"
@@ -153,6 +155,25 @@ func TestGetCurrentX509CASlot(t *testing.T) {
 	})
 }
 
+func TestCAPolicyIdentifiers(t *testing.T) {
+	ctx := context.Background()
+
+	test := setupTest(t)
+	test.initSelfSignedManager()
+	policy, err := x509.ParseOID("1.2.3.4")
+	require.NoError(t, err)
+	test.cc.policies = append(test.cc.policies, policy)
+
+	t.Run("contains policy identifiers", func(t *testing.T) {
+		require.NoError(t, test.m.PrepareX509CA(ctx))
+
+		currentSlot := test.m.GetCurrentX509CASlot()
+		slot := currentSlot.(*x509CASlot)
+		require.NotNil(t, slot.x509CA)
+		require.Equal(t, slot.x509CA.Certificate.Policies, test.cc.policies)
+	})
+}
+
 func TestGetNextX509CASlot(t *testing.T) {
 	ctx := context.Background()
 
@@ -457,11 +478,14 @@ func TestUpstreamProcessTaintedAuthorityBackoff(t *testing.T) {
 		}
 	}
 
+	test.clock.WaitForAfter(time.Second, "waiting for the retry to wait for next duration")
 	// Must fail due to the invalid key type
 	expectBackoffErr(t)
 
 	// Try again; expect to fail
 	test.clock.Add(6 * time.Second)
+
+	test.clock.WaitForAfter(time.Second, "waiting for the retry to wait for next duration")
 	expectBackoffErr(t)
 
 	// Restore to a valid key type, and advance time again
@@ -934,7 +958,6 @@ func TestPruneCAJournals(t *testing.T) {
 
 	var expectedCAJournals []*datastore.CAJournal
 	for _, testCase := range testCases {
-		testCase := testCase
 		expectedCAJournals = []*datastore.CAJournal{}
 		t.Run(testCase.name, func(t *testing.T) {
 			// Have a fresh data store in each test case
@@ -1139,7 +1162,6 @@ func TestAlternateKeyTypes(t *testing.T) {
 	}
 
 	for _, testCase := range testCases {
-		testCase := testCase
 		t.Run(testCase.name, func(t *testing.T) {
 			ctx := context.Background()
 
@@ -1205,6 +1227,7 @@ type managerTest struct {
 	km      keymanager.KeyManager
 	ds      *fakedatastore.DataStore
 	cat     *fakeservercatalog.Catalog
+	cc      fakeCC
 
 	m *Manager
 }
@@ -1294,10 +1317,11 @@ func (m *managerTest) selfSignedConfig() Config {
 
 func (m *managerTest) selfSignedConfigWithKeyTypes(x509CAKeyType, jwtKeyType keymanager.KeyType) Config {
 	credBuilder, err := credtemplate.NewBuilder(credtemplate.Config{
-		TrustDomain:   testTrustDomain,
-		X509CASubject: pkix.Name{CommonName: "SPIRE"},
-		Clock:         m.clock,
-		X509CATTL:     testCATTL,
+		TrustDomain:         testTrustDomain,
+		X509CASubject:       pkix.Name{CommonName: "SPIRE"},
+		Clock:               m.clock,
+		X509CATTL:           testCATTL,
+		CredentialComposers: []credentialcomposer.CredentialComposer{&m.cc},
 	})
 	require.NoError(m.t, err)
 
@@ -1546,3 +1570,30 @@ func (s *fakeCA) SetJWTKey(jwtKey *ca.JWTKey) {
 func (s *fakeCA) NotifyTaintedX509Authorities(taintedAuthorities []*x509.Certificate) {
 	s.taintedAuthoritiesCh <- taintedAuthorities
 }
+
+type fakeCC struct {
+	catalog.PluginInfo
+
+	policies []x509.OID
+}
+
+func (cc fakeCC) ComposeServerX509CA(_ context.Context, attributes credentialcomposer.X509CAAttributes) (credentialcomposer.X509CAAttributes, error) {
+	attributes.Policies = append(attributes.Policies, cc.policies...)
+	return attributes, nil
+}
+
+func (cc fakeCC) ComposeServerX509SVID(_ context.Context, attributes credentialcomposer.X509SVIDAttributes) (credentialcomposer.X509SVIDAttributes, error) {
+	return attributes, nil
+}
+
+func (cc fakeCC) ComposeAgentX509SVID(_ context.Context, _ spiffeid.ID, _ crypto.PublicKey, attributes credentialcomposer.X509SVIDAttributes) (credentialcomposer.X509SVIDAttributes, error) {
+	return attributes, nil
+}
+
+func (cc fakeCC) ComposeWorkloadX509SVID(_ context.Context, _ spiffeid.ID, _ crypto.PublicKey, attributes credentialcomposer.X509SVIDAttributes) (credentialcomposer.X509SVIDAttributes, error) {
+	return attributes, nil
+}
+
+func (cc fakeCC) ComposeWorkloadJWTSVID(_ context.Context, _ spiffeid.ID, attributes credentialcomposer.JWTSVIDAttributes) (credentialcomposer.JWTSVIDAttributes, error) {
+	return attributes, nil
+}
