@@ -3,7 +3,6 @@ package sqlstore
 import (
 	"errors"
 	"fmt"
-	"math"
 	"strconv"
 
 	"github.com/blang/semver/v4"
@@ -248,6 +247,13 @@ import (
 // | v1.10.4 |        |                                                                           |
 // |*********|********|***************************************************************************|
 // | v1.11.0 |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.11.1 |        |                                                                           |
+// |---------|        |                                                                           |
+// | v1.11.2 |        |                                                                           |
+// |*********|********|***************************************************************************|
+// | v1.12.0 |        |                                                                           |
+// | v1.12.1 |        |                                                                           |
 // ================================================================================================
 
 const (
@@ -269,12 +275,12 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 	// version before continuing, and fail if we're not.
 	if codeVersion.Major > 1 {
 		log.Error("Migration code needs updating for current release version")
-		return sqlError.New("current migration code not compatible with current release version")
+		return newSQLError("current migration code not compatible with current release version")
 	}
 
 	isNew := !db.HasTable(&Migration{})
 	if err := db.Error; err != nil {
-		return sqlError.Wrap(err)
+		return newWrappedSQLError(err)
 	}
 
 	if isNew {
@@ -283,12 +289,12 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 
 	// ensure migrations table exists so we can check versioning in all cases
 	if err := db.AutoMigrate(&Migration{}).Error; err != nil {
-		return sqlError.Wrap(err)
+		return newWrappedSQLError(err)
 	}
 
 	migration := new(Migration)
 	if err := db.Assign(Migration{}).FirstOrCreate(migration).Error; err != nil {
-		return sqlError.Wrap(err)
+		return newWrappedSQLError(err)
 	}
 
 	schemaVersion := migration.Version
@@ -298,7 +304,7 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 	dbCodeVersion, err := getDBCodeVersion(*migration)
 	if err != nil {
 		log.WithError(err).Error("Error getting DB code version")
-		return sqlError.New("error getting DB code version: %v", err)
+		return newSQLError("error getting DB code version: %v", err)
 	}
 
 	log = log.WithField(telemetry.VersionInfo, dbCodeVersion.String())
@@ -314,7 +320,7 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 			}
 
 			if err := db.Model(&Migration{}).Updates(newMigration).Error; err != nil {
-				return sqlError.Wrap(err)
+				return newWrappedSQLError(err)
 			}
 		}
 		return nil
@@ -323,7 +329,7 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 	if disableMigration {
 		if err = isDisabledMigrationAllowed(codeVersion, dbCodeVersion); err != nil {
 			log.WithError(err).Error("Auto-migrate must be enabled")
-			return sqlError.Wrap(err)
+			return newWrappedSQLError(err)
 		}
 		return nil
 	}
@@ -334,7 +340,7 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 	if schemaVersion > latestSchemaVersion {
 		if !isCompatibleCodeVersion(codeVersion, dbCodeVersion) {
 			log.Error("Incompatible DB schema is too new for code version, upgrade SPIRE Server")
-			return sqlError.New("incompatible DB schema and code version")
+			return newSQLError("incompatible DB schema and code version")
 		}
 		log.Warn("DB schema is ahead of code version, upgrading SPIRE Server is recommended")
 		return nil
@@ -348,7 +354,7 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 	for schemaVersion < latestSchemaVersion {
 		tx := db.Begin()
 		if err := tx.Error; err != nil {
-			return sqlError.Wrap(err)
+			return newWrappedSQLError(err)
 		}
 		schemaVersion, err = migrateVersion(tx, schemaVersion, log)
 		if err != nil {
@@ -356,7 +362,7 @@ func migrateDB(db *gorm.DB, dbType string, disableMigration bool, log logrus.Fie
 			return err
 		}
 		if err := tx.Commit().Error; err != nil {
-			return sqlError.Wrap(err)
+			return newWrappedSQLError(err)
 		}
 	}
 
@@ -387,19 +393,16 @@ func getDBCodeVersion(migration Migration) (dbCodeVersion semver.Version, err er
 }
 
 func isCompatibleCodeVersion(thisCodeVersion, dbCodeVersion semver.Version) bool {
-	// If major version is the same and minor version is +/- 1, versions are
-	// compatible
-	if dbCodeVersion.Major != thisCodeVersion.Major || (math.Abs(float64(int64(dbCodeVersion.Minor)-int64(thisCodeVersion.Minor))) > 1) {
-		return false
-	}
-	return true
+	// If major version is the same and minor version is +/- 1, versions are compatible
+	minMinor, maxMinor := min(dbCodeVersion.Minor, thisCodeVersion.Minor), max(dbCodeVersion.Minor, thisCodeVersion.Minor)
+	return dbCodeVersion.Major == thisCodeVersion.Major && (minMinor == maxMinor || minMinor+1 == maxMinor)
 }
 
 func initDB(db *gorm.DB, dbType string, log logrus.FieldLogger) (err error) {
 	log.Info("Initializing new database")
 	tx := db.Begin()
 	if err := tx.Error; err != nil {
-		return sqlError.Wrap(err)
+		return newWrappedSQLError(err)
 	}
 
 	tables := []any{
@@ -419,7 +422,7 @@ func initDB(db *gorm.DB, dbType string, log logrus.FieldLogger) (err error) {
 
 	if err := tableOptionsForDialect(tx, dbType).AutoMigrate(tables...).Error; err != nil {
 		tx.Rollback()
-		return sqlError.Wrap(err)
+		return newWrappedSQLError(err)
 	}
 
 	if err := tx.Assign(Migration{
@@ -427,7 +430,7 @@ func initDB(db *gorm.DB, dbType string, log logrus.FieldLogger) (err error) {
 		CodeVersion: codeVersion.String(),
 	}).FirstOrCreate(&Migration{}).Error; err != nil {
 		tx.Rollback()
-		return sqlError.Wrap(err)
+		return newWrappedSQLError(err)
 	}
 
 	if err := addFederatedRegistrationEntriesRegisteredEntryIDIndex(tx); err != nil {
@@ -435,7 +438,7 @@ func initDB(db *gorm.DB, dbType string, log logrus.FieldLogger) (err error) {
 	}
 
 	if err := tx.Commit().Error; err != nil {
-		return sqlError.Wrap(err)
+		return newWrappedSQLError(err)
 	}
 
 	return nil
@@ -459,11 +462,11 @@ func migrateVersion(tx *gorm.DB, currVersion int, log logrus.FieldLogger) (versi
 		Version:     nextVersion,
 		CodeVersion: version.Version(),
 	}).Error; err != nil {
-		return 0, sqlError.Wrap(err)
+		return 0, newWrappedSQLError(err)
 	}
 
 	if currVersion < lastMinorReleaseSchemaVersion {
-		return 0, sqlError.New("migrating from schema version %d requires a previous SPIRE release; please follow the upgrade strategy at doc/upgrading.md", currVersion)
+		return 0, newSQLError("migrating from schema version %d requires a previous SPIRE release; please follow the upgrade strategy at doc/upgrading.md", currVersion)
 	}
 
 	// Place all migrations handled by the current minor release here. This
@@ -487,7 +490,7 @@ func migrateVersion(tx *gorm.DB, currVersion int, log logrus.FieldLogger) (versi
 	//
 	switch currVersion { //nolint: gocritic // No upgrade required yet, keeping switch for future additions
 	default:
-		err = sqlError.New("no migration support for unknown schema version %d", currVersion)
+		err = newSQLError("no migration support for unknown schema version %d", currVersion)
 	}
 	if err != nil {
 		return 0, err
@@ -504,7 +507,7 @@ func addFederatedRegistrationEntriesRegisteredEntryIDIndex(tx *gorm.DB) error {
 	// to introduce the index since there is no explicit struct to add tags to
 	// so we have to manually create it.
 	if err := tx.Table("federated_registration_entries").AddIndex("idx_federated_registration_entries_registered_entry_id", "registered_entry_id").Error; err != nil {
-		return sqlError.Wrap(err)
+		return newWrappedSQLError(err)
 	}
 	return nil
 }
