@@ -80,7 +80,6 @@ func TestConfigure(t *testing.T) {
 	}
 	cases = append(cases, configureCasesOS(t)...)
 	for _, tt := range cases {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			var err error
 
@@ -125,6 +124,7 @@ func TestConfigure(t *testing.T) {
 }
 
 func TestMintX509CA(t *testing.T) {
+	mockClock := clock.NewMock(t)
 	ca := testca.New(t, trustDomain)
 
 	// Create SVID returned when fetching
@@ -261,10 +261,7 @@ func TestMintX509CA(t *testing.T) {
 
 	cases = append(cases, mintX509CACasesOS(t)...)
 	for _, c := range cases {
-		c := c
 		t.Run(c.name, func(t *testing.T) {
-			mockClock := clock.NewMock(t)
-
 			// Setup servers
 			server := testHandler{}
 			server.startTestServers(t, mockClock, ca, serverCert, serverKey, svidCert, svidKey)
