@@ -1,13 +1,15 @@
 package main
 
 import (
+	"errors"
+	"fmt"
 	"net"
 	"net/url"
 	"os"
 	"time"
 
 	"github.com/hashicorp/hcl"
-	"github.com/zeebo/errs"
+	"github.com/spiffe/spire/pkg/common/config"
 )
 
 const (
@@ -79,6 +81,15 @@ type Config struct {
 
 	// JWTIssuer specifies the issuer for the OIDC provider configuration request.
 	JWTIssuer string `hcl:"jwt_issuer"`
+
+	// JWKSURI specifies the absolute uri to the jwks keys document. Use this if you are fronting the
+	// discovery provider with a load balancer or reverse proxy
+	JWKSURI string `hcl:"jwks_uri"`
+
+	// ServerPathPrefix specifies the prefix to strip from the path of requests to route to the server.
+	// Example: if ServerPathPrefix is /foo then a request to http://127.0.0.1/foo/.well-known/openid-configuration and
+	// http://127.0.0.1/foo/keys will function with the server.
+	ServerPathPrefix string `hcl:"server_path_prefix"`
 }
 
 type ServingCertFileConfig struct {
@@ -185,18 +196,22 @@ type experimentalWorkloadAPIConfig struct {
 	NamedPipeName string `hcl:"named_pipe_name" json:"named_pipe_name"`
 }
 
-func LoadConfig(path string) (*Config, error) {
+func LoadConfig(path string, expandEnv bool) (*Config, error) {
 	hclBytes, err := os.ReadFile(path)
 	if err != nil {
-		return nil, errs.New("unable to load configuration: %v", err)
+		return nil, fmt.Errorf("unable to load configuration: %w", err)
 	}
-	return ParseConfig(string(hclBytes))
+	hclString := string(hclBytes)
+	if expandEnv {
+		hclString = config.ExpandEnv(hclString)
+	}
+	return ParseConfig(hclString)
 }
 
 func ParseConfig(hclConfig string) (_ *Config, err error) {
 	c := new(Config)
 	if err := hcl.Decode(c, hclConfig); err != nil {
-		return nil, errs.New("unable to decode configuration: %v", err)
+		return nil, fmt.Errorf("unable to decode configuration: %w", err)
 	}
 
 	if c.LogLevel == "" {
@@ -204,7 +219,7 @@ func ParseConfig(hclConfig string) (_ *Config, err error) {
 	}
 
 	if len(c.Domains) == 0 {
-		return nil, errs.New("at least one domain must be configured")
+		return nil, errors.New("at least one domain must be configured")
 	}
 	c.Domains = dedupeList(c.Domains)
 
@@ -215,20 +230,20 @@ func ParseConfig(hclConfig string) (_ *Config, err error) {
 		}
 		switch {
 		case c.InsecureAddr != "":
-			return nil, errs.New("insecure_addr and the acme section are mutually exclusive")
+			return nil, errors.New("insecure_addr and the acme section are mutually exclusive")
 		case !c.ACME.ToSAccepted:
-			return nil, errs.New("tos_accepted must be set to true in the acme configuration section")
+			return nil, errors.New("tos_accepted must be set to true in the acme configuration section")
 		case c.ACME.Email == "":
-			return nil, errs.New("email must be configured in the acme configuration section")
+			return nil, errors.New("email must be configured in the acme configuration section")
 		}
 	}
 
 	if c.ServingCertFile != nil {
 		if c.ServingCertFile.CertFilePath == "" {
-			return nil, errs.New("cert_file_path must be configured in the serving_cert_file configuration section")
+			return nil, errors.New("cert_file_path must be configured in the serving_cert_file configuration section")
 		}
 		if c.ServingCertFile.KeyFilePath == "" {
-			return nil, errs.New("key_file_path must be configured in the serving_cert_file configuration section")
+			return nil, errors.New("key_file_path must be configured in the serving_cert_file configuration section")
 		}
 
 		if c.ServingCertFile.RawAddr == "" {
@@ -237,13 +252,13 @@ func ParseConfig(hclConfig string) (_ *Config, err error) {
 
 		addr, err := net.ResolveTCPAddr("tcp", c.ServingCertFile.RawAddr)
 		if err != nil {
-			return nil, errs.New("invalid addr in the serving_cert_file configuration section: %v", err)
+			return nil, fmt.Errorf("invalid addr in the serving_cert_file configuration section: %w", err)
 		}
 		c.ServingCertFile.Addr = addr
 
 		c.ServingCertFile.FileSyncInterval, err = parseDurationField(c.ServingCertFile.RawFileSyncInterval, defaultFileSyncInterval)
 		if err != nil {
-			return nil, errs.New("invalid file_sync_interval in the serving_cert_file configuration section: %v", err)
+			return nil, fmt.Errorf("invalid file_sync_interval in the serving_cert_file configuration section: %w", err)
 		}
 	}
 
@@ -252,18 +267,18 @@ func ParseConfig(hclConfig string) (_ *Config, err error) {
 	if c.ServerAPI != nil {
 		c.ServerAPI.PollInterval, err = parseDurationField(c.ServerAPI.RawPollInterval, defaultPollInterval)
 		if err != nil {
-			return nil, errs.New("invalid poll_interval in the server_api configuration section: %v", err)
+			return nil, fmt.Errorf("invalid poll_interval in the server_api configuration section: %w", err)
 		}
 		methodCount++
 	}
 
 	if c.WorkloadAPI != nil {
 		if c.WorkloadAPI.TrustDomain == "" {
-			return nil, errs.New("trust_domain must be configured in the workload_api configuration section")
+			return nil, errors.New("trust_domain must be configured in the workload_api configuration section")
 		}
 		c.WorkloadAPI.PollInterval, err = parseDurationField(c.WorkloadAPI.RawPollInterval, defaultPollInterval)
 		if err != nil {
-			return nil, errs.New("invalid poll_interval in the workload_api configuration section: %v", err)
+			return nil, fmt.Errorf("invalid poll_interval in the workload_api configuration section: %w", err)
 		}
 		methodCount++
 	}
@@ -286,17 +301,31 @@ func ParseConfig(hclConfig string) (_ *Config, err error) {
 
 	switch methodCount {
 	case 0:
-		return nil, errs.New("either the server_api or workload_api section must be configured")
+		return nil, errors.New("either the server_api or workload_api section must be configured")
 	case 1:
 	default:
-		return nil, errs.New("the server_api and workload_api sections are mutually exclusive")
+		return nil, errors.New("the server_api and workload_api sections are mutually exclusive")
 	}
 	if c.JWTIssuer != "" {
 		jwtIssuer, err := url.Parse(c.JWTIssuer)
-		if err != nil || jwtIssuer.Scheme == "" || jwtIssuer.Host == "" {
-			return nil, errs.New("the jwt_issuer url could not be parsed")
+		switch {
+		case err != nil:
+			return nil, fmt.Errorf("the jwt_issuer url could not be parsed: %w", err)
+		case jwtIssuer.Scheme == "":
+			return nil, errors.New("the jwt_issuer url must contain a scheme")
+		case jwtIssuer.Host == "":
+			return nil, errors.New("the jwt_issuer url must contain a host")
 		}
 	}
+	if c.JWKSURI != "" {
+		jwksURI, err := url.Parse(c.JWKSURI)
+		if err != nil || jwksURI.Scheme == "" || jwksURI.Host == "" {
+			return nil, fmt.Errorf("the jwks_uri setting could not be parsed: %w", err)
+		}
+	}
+	if c.JWKSURI == "" && c.JWTIssuer != "" {
+		fmt.Printf("Warning: The jwt_issuer configuration will also affect the jwks_uri behavior when jwks_url is not set. This behaviour will be changed in 1.13.0.")
+	}
 	return c, nil
 }
 
