@@ -3,28 +3,31 @@ package main
 import (
 	"context"
 	"crypto/tls"
+	"errors"
 	"flag"
 	"fmt"
 	"net"
 	"net/http"
+	"net/url"
 	"os"
 	"os/signal"
 	"syscall"
 	"time"
 
 	"github.com/sirupsen/logrus"
+	"golang.org/x/crypto/acme"
+	"golang.org/x/crypto/acme/autocert"
+
 	"github.com/spiffe/spire/pkg/common/diskcertmanager"
 	"github.com/spiffe/spire/pkg/common/log"
 	"github.com/spiffe/spire/pkg/common/telemetry"
 	"github.com/spiffe/spire/pkg/common/version"
-	"github.com/zeebo/errs"
-	"golang.org/x/crypto/acme"
-	"golang.org/x/crypto/acme/autocert"
 )
 
 var (
 	versionFlag = flag.Bool("version", false, "print version")
 	configFlag  = flag.String("config", "oidc-discovery-provider.conf", "configuration file")
+	expandEnv   = flag.Bool("expandEnv", false, "expand environment variables in config file")
 )
 
 func main() {
@@ -35,21 +38,21 @@ func main() {
 		os.Exit(0)
 	}
 
-	if err := run(*configFlag); err != nil {
+	if err := run(*configFlag, *expandEnv); err != nil {
 		fmt.Fprintf(os.Stderr, "%+v\n", err)
 		os.Exit(1)
 	}
 }
 
-func run(configPath string) error {
-	config, err := LoadConfig(configPath)
+func run(configPath string, expandEnv bool) error {
+	config, err := LoadConfig(configPath, expandEnv)
 	if err != nil {
 		return err
 	}
 
 	log, err := log.NewLogger(log.WithLevel(config.LogLevel), log.WithFormat(config.LogFormat), log.WithOutputFile(config.LogPath))
 	if err != nil {
-		return errs.Wrap(err)
+		return err
 	}
 	defer log.Close()
 
@@ -67,7 +70,23 @@ func run(configPath string) error {
 		return err
 	}
 
-	var handler http.Handler = NewHandler(log, domainPolicy, source, config.AllowInsecureScheme, config.SetKeyUse, config.JWTIssuer)
+	var jwtIssuer *url.URL
+	if config.JWTIssuer != "" {
+		jwtIssuer, err = url.Parse(config.JWTIssuer)
+		if err != nil {
+			return err
+		}
+	}
+
+	var jwksURI *url.URL
+	if config.JWKSURI != "" {
+		jwksURI, err = url.Parse(config.JWKSURI)
+		if err != nil {
+			return err
+		}
+	}
+
+	var handler http.Handler = NewHandler(log, domainPolicy, source, config.AllowInsecureScheme, config.SetKeyUse, jwtIssuer, jwksURI, config.ServerPathPrefix)
 	if config.LogRequests {
 		log.Info("Logging all requests")
 		handler = logHandler(log, handler)
@@ -157,7 +176,7 @@ func newSource(log logrus.FieldLogger, config *Config) (JWKSSource, error) {
 	case config.WorkloadAPI != nil:
 		workloadAPIAddr, err := config.getWorkloadAPIAddr()
 		if err != nil {
-			return nil, errs.Wrap(err)
+			return nil, err
 		}
 		return NewWorkloadAPISource(WorkloadAPISourceConfig{
 			Log:          log,
@@ -167,7 +186,7 @@ func newSource(log logrus.FieldLogger, config *Config) (JWKSSource, error) {
 		})
 	default:
 		// This is defensive; LoadConfig should prevent this from happening.
-		return nil, errs.New("no source has been configured")
+		return nil, errors.New("no source has been configured")
 	}
 }
 
@@ -231,7 +250,7 @@ func logHandler(log logrus.FieldLogger, handler http.Handler) http.Handler {
 			"remote-addr": r.RemoteAddr,
 			"method":      r.Method,
 			"url":         r.URL,
-			"user-agent":  r.UserAgent,
+			"user-agent":  r.UserAgent(),
 		}).Debug("Incoming request")
 		handler.ServeHTTP(w, r)
 	})
