@@ -3,8 +3,9 @@ package fakedatastore
 import (
 	"context"
 	"fmt"
+	"net/url"
+	"path/filepath"
 	"sort"
-	"sync/atomic"
 	"testing"
 	"time"
 
@@ -21,8 +22,6 @@ import (
 
 var (
 	ctx = context.Background()
-
-	nextID uint32
 )
 
 type DataStore struct {
@@ -38,12 +37,20 @@ func New(tb testing.TB) *DataStore {
 	ds := sql.New(log)
 	ds.SetUseServerTimestamps(true)
 
+	tmpDir := tb.TempDir()
+	dbPath := filepath.Join(tmpDir, "spire.db")
+	dbPath = url.PathEscape(dbPath)
+
 	err := ds.Configure(ctx, fmt.Sprintf(`
 		database_type = "sqlite3"
-		connection_string = "file:memdb%d?mode=memory&cache=shared"
-	`, atomic.AddUint32(&nextID, 1)))
+		connection_string = "file:%s"
+	`, dbPath))
 	require.NoError(tb, err)
 
+	tb.Cleanup(func() {
+		ds.Close()
+	})
+
 	return &DataStore{
 		ds: ds,
 	}
@@ -279,6 +286,13 @@ func (s *DataStore) FetchRegistrationEntry(ctx context.Context, entryID string)
 	return s.ds.FetchRegistrationEntry(ctx, entryID)
 }
 
+func (s *DataStore) FetchRegistrationEntries(ctx context.Context, entryIDs []string) (map[string]*common.RegistrationEntry, error) {
+	if err := s.getNextError(); err != nil {
+		return nil, err
+	}
+	return s.ds.FetchRegistrationEntries(ctx, entryIDs)
+}
+
 func (s *DataStore) ListRegistrationEntries(ctx context.Context, req *datastore.ListRegistrationEntriesRequest) (*datastore.ListRegistrationEntriesResponse, error) {
 	if err := s.getNextError(); err != nil {
 		return nil, err
