@@ -49,7 +49,6 @@ func New(t *testing.T, trustDomain spiffeid.TrustDomain, ds datastore.DataStore,
 	service := entry.New(entry.Config{
 		TrustDomain: trustDomain,
 		DataStore:   ds,
-		//EntryFetcher: authorizedEntryFetcherWithCache,
 	})
 
 	log, _ := test.NewNullLogger()
@@ -68,7 +67,7 @@ func New(t *testing.T, trustDomain spiffeid.TrustDomain, ds datastore.DataStore,
 		grpctest.Credentials(fakeTransportCreds{}),
 	)
 
-	conn := server.Dial(t)
+	conn := server.NewGRPCClient(t)
 
 	c.done = server.Stop
 	c.EntryClient = entryv1.NewEntryClient(conn)
